﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Encodings.RealVectorEncoding;

// ReSharper disable once CheckNamespace
namespace HeuristicLab.Algorithms.EGO {

    [StorableType("5e725905-0d77-4379-bbb9-8ca6ae392e59")]
    [Item("UniformRandomSampling", "A uniform random sampling strategy for real valued optimization")]
  public class UniformRandomSampling : ParameterizedNamedItem, IInitialSampling<RealVector> {

    #region HL-Constructors, Serialization and Cloning 
    [StorableConstructor]
    protected UniformRandomSampling(StorableConstructorFlag deserializing) : base(deserializing) { }
    protected UniformRandomSampling(UniformRandomSampling original, Cloner cloner) : base(original, cloner) { }
    public UniformRandomSampling() {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new UniformRandomSampling(this, cloner);
    }

    public RealVector[] GetSamples(int noSamples, RealVector[] existingSamples, IEncoding encoding, IRandom random) {
      var enc = encoding as RealVectorEncoding;
      var res = new RealVector[noSamples];
      for (var i = 0; i < noSamples; i++) {
        var r = new RealVector(enc.Length);
        res[i] = r;
        for (var j = 0; j < enc.Length; j++) {
          var b = j % enc.Bounds.Rows;
          r[j] = UniformRandom(enc.Bounds[b, 0], enc.Bounds[b, 1], random);
        }
      }
      return res;
    }
    private static double UniformRandom(double min, double max, IRandom rand) {
      return rand.NextDouble() * (max - min) + min;
    }
  }




  #endregion


}

